.module/ram/abs=0      C1TBOX;
.pagelength  59;                    { Works for .LST files with 550C }
.include    <system.k>;

{******************************************************************************
 *
 *      EZ-KIT Lite DSP Program
 *	c1tbox.dsp  -  Test Box - Generates 2 sine waves + noise.
 *      Uses EZ-Kit Lite and Knob Box.
 *	Bob Larkin, W7PUA     15 Jan 2000
 *
 * A general purpose test box to  generate one sine wave, 2 sine waves
 * added together, or these plus Gaussian random noise.
 *
 * This includes the subroutines for interacting with the Knob/Display
 * box (see c1knob.dsp for more details).
 *
 * Button 1   Function scroll (tone 1, tone 2, noise)
 * Button 2   Frequency Set if pushed, Amplitude set if up
 * Button 3   Not Used
 * Button 4   Toggle all output on and off
 *
 * Green LED  Power On
 * Red LED    Output on
 *
 *****************************************************************************}

{***********************   CONSTANTS     *************************************}
.const		TIME1 = 4000;
.const		TIME2 = 510;

.const		SIN_LEV=3277;		{ 0.1 in 1.15}
.const		SIN_DPH=1365;		{ Phase for 1000.24 Hz }
.const		RN_SEED=0x1234;		{ This can be anything }
.const		FR2PH=0X5762;		{ Cnvt freq to phase increment-DDS }

{********************   PROGRAM MEMORY     ***********************************}
.var/pm         sin_coeff[5];
.init           sin_coeff: H#324000, H#005300, H#AACC00, H#08B700, H#1CCE00;

.var/pm	encoder[16];			{ Last state*4 + New state is adr and
					  output is the implied change }
.init	encoder: 0, H#FFFF00, H#000100, 0,
		 H#000100, 0, 0, H#FFFF00,
		 H#FFFF00, 0, 0, H#000100,
		 0, H#000100, H#FFFF00, 0;
{ Note - the encoder table is stored as 24 bit data in pm, but used as
  16 bit data in the dsp. The 00's on the end of the hex values are 8
  bits, set to 0, that are never used, but are very necessary to make
  the bits line up when read as 16 bit values.   }

{********************   DATA MEMORY     **************************************}
{ SPORT0, i.e., Serial PORT 0, is located in data memory.
  The AD1847 Codec hardware operates independently of the DSP.
  But when A/D conversion is complete there is an interrupt generated
  by SPORT0.  This occurs 48,000 times per second as determined
  by the crystal clock in the Codec.  This interrupt causes, among other
  things, a jump to location 14 Hex in program memory.  This, in turn,
  is sent off to process the newly received data, or to do any other task
  required on a periodic basis, such as generation of waveforms.
  It is at this time that new data can be sent to the Codec to be converted
  by the D/A in the Codec.  Memory locations tx_buf and rx_buf provide the
  interaction with the SPORT0 (and thus the Codec) when interrupts occur. }
.var/dm/circ	tx_buf[3];		{ SPORT0: Cmd + L data + R data }
.var/dm/circ 	rx_buf[3];		{ SPORT0: Status + L data + R data }

{ data16 is the 16 bits that will be clocked out to the hardware
  shift registers.  This is sent by the function load16.
  Many functions use data16.  Only modify the bits of interest.
	Bit 0		LSB of input address
	Bit 1		Middle bit of input address
	Bit 2		MSB of input address
	Bit 3		Green LED, 1=on
	Bit 4		Red LED, 1=on
	Bit 5		LCD Enable line
	Bit 6		LCD Read/Write Line, 0=Write
	Bit 7		LCD Data/Command line, 1=Data
	Bit 8		LCD Data, Bit 4
	BIt 9		LCD Data, Bit 5
	Bit 10		LCD Data, Bit 6
	BIt 11		LCD Data, Bit 7
	Bit 12 to 15	Spares, Not Used
}
.var/dm		data16;
.var/dm		sr_data;		{ Working register used by load16 }

{ Two software clocks are used to slow down the update rate of the LCD
  display and of the Switches.  This gives more time to update the knob
  data.  }
.var/dm		clock1, clock2;
.var/dm		knob_st;		{ Determines if knob moved since last}
.var/dm		which_fcn;
.var/dm		ampl_freq;		{ 0=amplitude, 1=freq }
.var/dm		off_on;			{ 0=Output Off, 1=On }
.var/dm		sw1, sw4;		{ Last state of toggle buttons }
.var/dm		temp1;			{ These are available for any fcn }
.var/dm		temp3;
.var/dm		number;
.var/dm		digit[6];		{ ASCII number, sign + 5 BCD  }
.var/dm		msgl[8];		{ Mirrors the LCD as needed; left 8 }
.var/dm		msgr[8];		{ Same, only right 8 characters }

{ Initialization of the AD1847 Codec is complex.  The internal Codec
  registers are transfered out of the following buffer.  It is circular to
  allow detection of a complete load of commands.     }
.var/dm/circ    init_cmds[13];		{ Codec Initialization }
.var/dm		stat_flag;		{ Codec Init status }

.var/dm		phase1;			{ Cumulated phase of sine wave #1 }
.var/dm		phase2;			{ Cumulated phase of sine wave #2 }
.var/dm		signal_save;
.var/dm		sig1_freq;
.var/dm		sig1_dphase;		{ DDS differential phase between sampl}
.var/dm		sig1_mult;		{ The actual multiplier, shift R 12 }
.var/dm		sig1_level_lo;		{ Gain is stored *4096 in 32 bit form }
.var/dm		sig1_level_hi;
.var/dm		sig1_level_mv;		{ 0 to 1750 in 0.1 mV steps }
.var/dm		sig2_freq;
.var/dm		sig2_dphase;
.var/dm		sig2_mult;
.var/dm		sig2_level_lo;
.var/dm		sig2_level_hi;
.var/dm		sig2_level_mv;
.var/dm		noise_mult;
.var/dm		noise_level_lo;
.var/dm		noise_level_hi;
.var/dm		noise_level_mv;

.var/dm		seed_msw;		{ The seed for random number }
.var/dm		seed_lsw;		{   generation, saved as 2x16 bits }

{********************  Interrupt vector table  *******************************}

	{ A jump to location 0 occurs when the dsp is reset }
        jump start;  rti; rti; rti;     {00: reset }

	{ Most of the interrupt vectors are not used, such as the next 3 }
        rti;         rti; rti; rti;     {04: IRQ2 }	{ Filler instr }
        rti;         rti; rti; rti;     {08: IRQL1 }
        rti;         rti; rti; rti;     {0c: IRQL0 }

	{ Do to the variable stat_flag, this interrupt routine is only used
	  during initialization of the AD1847 Codec.  It is used to send
	  register data to the Codec.  By using the interrupt the data is
	  not sent when the SPORT0 transmit buffer is full.         }
	ar = dm(stat_flag);		{10: SPORT0 tx }
        ar = pass ar;			{  see if needs initialization }
        if eq rti;			{ If dm(stat_flag)==0 exit this int }
        jump next_cmd;			{   else go transfer a control word }

	{ The following instructions are executed when an A/D conversion
	  cycle has been completed in the AD1847 Codec, and the resulting data
	  has been serially transmitted to the SPORT0.  The SPORT0 hardware
	  then generates an interrupt with an associated jump to this
	  program address: }
	jump input_samples;             {14: SPORT0 rx }
                     rti; rti; rti;	{Filler so that there are 4 instr }

	{ And more unused interrupt locations: }
        rti;         rti; rti; rti;     {18: IRQE }
        rti;         rti; rti; rti;     {1c: BDMA }
	rti;         rti; rti; rti;     {20: SPORT1 tx or IRQ1 }
        rti;         rti; rti; rti;     {24: SPORT1 rx or IRQ0 }
	rti;         rti; rti; rti;     {28: timer }
        rti;         rti; rti; rti;     {2c: power down }

{*****************************  START  ***************************************
 * Initialize process and drop into main loop.  None of the index registers
 * have been initialized, so we need to turn off all interrupts before
 * the hardware is allowed to generate one.  Then we are safe with instructions
 * like dm(i0, m0)=ar.  This is the dreaded uninitialized pointer problem!!
 ****************************************************************************}
start:	imask=0;			{ Turn off all interrupts }
	
	{ Initialization has been broken into two pieces to simplify testing.
	  Most items should go into init0, but those that involve interrupts
	  can be put into init1 so that they can be commented out easily
	  when using the simulator.  }
	call init0;			{ Instructions that simulate easily }

	call init1;			{ And those that do not }

	{ Initialize LCD and start a load }
	ax0 = 0;			{ Start with all hardware shift }
	dm(data16) = ax0;		{   register outputs reset to 0  }
	call load16;

	call init_lcd;			{ Init sequence to LCD }
	call knob;			{ Initialize knob state }
	call grn_on;			{ Power on}
	call red_on;			{ Output is enabled }

{****************     Main Program Loop    ************************************
 *  Main processing loop.  Does all non-periodic (background) processing.
 *  This loop is interrupted at a 48KHz rate to send and receive Codec data.
 *****************************************************************************}

again:	{Check state and sort out here for knob action }
	ay0=dm(which_fcn);		{ Test for sine #1, #2, or noise }
	none=pass ay0;			{ If zero... }
	if eq jump sin_1;		{ Go set sine wave #1 }
	ar=ay0-1;			{ If one...  }
	if eq jump sin_2;		{ Go set sine wave #2 }

	{ Here we adjust the noise amplitude if the knob has moved. We need to
	  alter both noise_level_mv for display, and the noise_mult, used
	  to multiply the noise output to achieve the correct level.
	  In order to maintain the precision as we add and subtract tenths
	  of a mV, this is maintained as a double precision number,
	  noise_level_lo and noise_level_hi. The conversion from double
	  precision to noise_mult is a shift of 12 bits.  }
	call knob;			{ See if knob has moved (in ay0) }
	if eq jump ag1;			{ Zero flag means no change }
	ar=dm(noise_level_mv);		{ Alter by either + or - 1 }
	ar=ar+ay0;			{ We add, but ay0 may be + or - 1 }
	dm(noise_level_mv)=ar;		{ Save for next time }
	mx0=ay0;			{ Need to alter the multiplier, also }
	my0=18981;			{ 0.1 mV * 4096 }
	mr=mx0*my0 (ss);		{ Get the +1 or -1 multiplier }
	ay1=dm(noise_level_lo);		{ The low 16 bits }
	ar=mr0+ay1;			{ Alter 0.1mV worth }
	sr0=ar;				{ Save for later shift }
	dm(noise_level_lo)=ar;		{ Save for next alteration }
	ay1=dm(noise_level_hi);		{ The high 16 bits }
	ar=mr1+ay1+c;			{ Alter them, c is still there }
	dm(noise_level_hi)=ar;		{ Save for next alteration }
	sr=ashift ar by -12 (hi);	{ Convert to noise_mult }
	dm(noise_mult)=sr0;
	jump ag1;			{ We are done for now }


sin_1:	ar=dm(ampl_freq);		{ 0 for ampl, 1 for freq }
	ar=pass ar;
	if gt jump sin_1f;
	{ Here we adjust the amplitude if the knob has moved, see the comments
	  above on noise level setting, it is the same }
	call knob;
	if eq jump ag1;
	ar=dm(sig1_level_mv);
	ar=ar+ay0;
	dm(sig1_level_mv)=ar;
	mx0=ay0;
	my0=18981;
	mr=mx0*my0 (ss);

	ay1=dm(sig1_level_lo);
	ar=mr0+ay1;
	sr0=ar;
	dm(sig1_level_lo)=ar;
	ay1=dm(sig1_level_hi);
	ar=mr1+ay1+c;
	dm(sig1_level_hi)=ar;
	sr=ashift ar by -12 (hi);
	dm(sig1_mult)=sr0;
	jump ag1;

	{ Button 3 is down, so we need to alter the frequency by 1 Hz, or
	  no change at all. The frequency in Hz is stored in sig1_freq. It is
	  converted to the differential phase value by multiplying by
	  a constant and then stored in sig1_dphase.  This latter quantity,
	  when divided by 65,536 is the fraction of 360 degrees that the
	  phase of the sine wave changes between samples. Since samples are
	  at a 48,000 /sec rate, the resolution of the frequency setting is
	  48/000/65536=0.73 Hz. }
sin_1f:	call knob;			{ See if knob has moved (in ay0) }
	if eq jump ag1;			{ Zero means the knob has not moved }
	ar=dm(sig1_freq);		{ Alter the freq stored in sig1_freq }
	ar=ar+ay0;			{   by + or - 1Hz }
	if ge jump s1fa;		{ Stop the changes at 0, no neg freq }
	ar=0;				{ If neg, make 0 }
s1fa:	dm(sig1_freq)=ar;		{ Save the altered freq }
	{ Frequency is in Hz sig1_freq, and is needed as a phase increment.
	  We need to multiply by 65536/48000, but in the 1.15 arithmetic, the
	  biggest value is 1.0. So we multiply by FR2PH=0.5*65536/48000 and
	  then shift left 1 bit, the same as a multiply by 2. }
	my0=FR2PH;
	mr=ar*my0 (ss);			{ The fractional multiply }
        sr=ashift mr1 by 1 (hi);	{   and the multiply by 2, which comes}
        sr=sr or lshift mr0 by 1 (lo);  {   in two parts to get the LS bit }
	dm(sig1_dphase)=sr1;		{ Save the phase increment for DDS }
	jump ag1;			{ Skip over sine wave 2 stuff }

	{ Same as above, but for sine wave #2. Comments on the instructions
	  are essentially the same as those above, and are not listed again. }
sin_2:	ar=dm(ampl_freq);
	ar=pass ar;
	if gt jump sin_2f;
	call knob;
	if eq jump ag1;
	ar=dm(sig2_level_mv);
	ar=ar+ay0;
	dm(sig2_level_mv)=ar;
	mx0=ay0;
	my0=18981;
	mr=mx0*my0 (ss);
	ay1=dm(sig2_level_lo);
	ar=mr0+ay1;
	sr0=ar;
	dm(sig2_level_lo)=ar;
	ay1=dm(sig2_level_hi);
	ar=mr1+ay1+c;
	dm(sig2_level_hi)=ar;
	sr=ashift ar by -12 (hi);
	dm(sig2_mult)=sr0;
	jump ag1;

sin_2f:	call knob;
	if eq jump ag1;
	ar=dm(sig2_freq);
	ar=ar+ay0;
	if ge jump s2fa;
	ar=0;
s2fa:	dm(sig2_freq)=ar;
	my0=FR2PH;
	mr=ar*my0 (ss);
        sr=ashift mr1 by 1 (hi);
        sr=sr or lshift mr0 by 1 (lo);
	dm(sig2_dphase)=sr1;

ag1:	{ clock1 and clock2 decrease by 1 every 20.8 usec in input_samples }
	{ Now, see if clock1 is to zero.  If so, display number. }
	ar = dm(clock1);
	ar = pass ar;
	if gt jump ag2;			{ Not time yet }
	ar = TIME1;			{ Reset clock1 to TIME1 }
	dm(clock1) = ar;

	ay0=dm(which_fcn);		{ Test for sine #1, #2, or noise }
	none=pass ay0;			{ If zero... }
	if eq jump show_1;		{ Go set sine wave #1 }
	ar=ay0-1;			{ If one...  }
	if eq jump show_2;		{ Go set sine wave #2 }

	{Stay here and display noise parameters. For simplicity, the frequency
	 is still displayed, but set to 0. }
	ax0=H#004E;			{ The letter 'N' for noise }
	ar=0;				{ 0 freq for noise }
	call send_freq;			{ Displays the left 8 characters }
	ar=dm(noise_level_mv);		{ Get the rms mV of the noise }
	call send_lev;			{ Display in the right 8 characters }
	jump ag2;

	{ The following display the two sine-wave parameters. See above for
	  detailed comments. }
show_1:	ax0=H#0031;
	ar=dm(sig1_freq);
	call send_freq;
	ar=dm(sig1_level_mv);
	call send_lev;
	jump ag2;

show_2:	ax0=H#0032;
	ar=dm(sig2_freq);
	call send_freq;
	ar=dm(sig2_level_mv);
	call send_lev;

	{ Now, see if clock2 is to zero.  If so, update switches. }
ag2:	ar = dm(clock2);		{ Get clock2 count, set at interrupt}
	ar = pass ar;			{ Test the value for zero }
	if gt jump ag4;			{ Ignore, not zero }
	ar = TIME2;			{ Zero, reset to TIME2 for next }
	dm(clock2) = ar;		{   count down }

	{ Look at all buttons and alter variables to suit there settings }
ag4:	call in_sw4;			{ This scrolls through the 3 signals }
	if ne jump sw4b;		{ Jump if button is up }
	ar=dm(sw4);			{ Get last state (0=up) }
	ar=pass ar;			{ Test last state }
	if gt jump sw4c;		{ Do nothing if was down last time }
	ay0=dm(which_fcn);		{ Here to toggle and change fcn }
	ar=ay0+1;			{ Move to next fcn }
	ay0=3;				{ Highest fcn is 2 (noise) }
	none=ar-ay0;			{ Test if time to wrap around }
	if lt jump sw4a;		{ Jump if not }
	ar=0;				{ Go wrap around }
sw4a:	dm(which_fcn)=ar;		{ Save for future use }
	ax0=1;				{ 1 marks the button as down }
	jump sw4c;
sw4b:	ax0=0;				{ 0 marks the button as up }
sw4c:	dm(sw4)=ax0;			{ For use next time }

	call in_sw3;			{ This determines if freq or ampl }
	ar=0;
	if ne jump sw3a;		{ Jump if button is up (ampl) }
	ar=1;
sw3a:	dm(ampl_freq)=ar;		{ Here for freq }

	{ SW2 is not used
	call in_sw2;
	if ne jump sw2a;
	}
sw2a:
	call in_sw1;			{ This switch toggles on and off }
	if ne jump sw1b;		{ Do nothing the button is up }
	{ Here because button 1 is down }
	ar=dm(sw1);			{ Get last state (0=up) }
	ar=pass ar;			{ Test last state }
	if gt jump sw1c;		{ Do nothing if was down last time }
	{ Here because button 1 is down, and it wasn't last time. So we
	  need to toggle off/on, control Red LED, and keep track of states. }
	ax0=1;
	dm(sw1)=ax0;			{ Mark the button down, for next time }
	ax0=dm(off_on);			{ Here to toggle output off/on }
	ay0=1;				{ Pick out bit 0 }
	ar=ax0 xor ay0;			{ Toggle the off/on state }
	dm(off_on)=ar;			{ Save to control signals }
	if eq jump sw1a;		{ Jump if state is off }
	{ Here because the state is "On" }
	call red_on;			{ Turn LED on }
	jump sw1c;
sw1a:	call red_off;			{ Turn LED off }
	jump sw1c;
sw1b:	ax0=0;				{ 0 marks the button as up }
	dm(sw1)=ax0;			{ For use next time }
sw1c:
	jump again;			{ Go round and round forever }

{***********************************************************************
 *                 SPORT0 Interrupt Handler                            *
 *  SPORT0 inputs from the Codec put us here 48000 times per second.   *
 *  Routines here are those that need this periodic service.           *
 ***********************************************************************}
input_samples:

	{ We do not know where calculations were at when the interrupt was
	  executed.  All of the processor registers that are used by the
	  background process must be protected, if they are to be used
	  by the interrupting routine, where we are now.  This is done by
	  having two sets of most registers and switching them with
	  the "ena sec_reg" instruction.  Note that the i index registers,
	  and the associated m and l registers are not switched.  There
	  is only one set of these 24 registers.   }
        ena sec_reg;                    { use secondary register bank }

	ay0 = dm(clock1);		{ Slow down counter for LCD update }
	ar = ay0-1;			{ Count down from TIME1 to 0 }
	if lt jump is8;			{ Don't go below 0 }
	dm(clock1) = ar;		{ Save for next interrupt }

is8:	ay0 = dm(clock2);		{ Slow down counter for Switch update }
	ar = ay0-1;			{ Decrement the count }
	if lt jump is9;			{ But not less than zero }
	dm(clock2) = ar;		{ Save if >= zero }
is9:
	{ For details on the generation of the sine wave, see c1sin.dsp }
	ax1=dm(sig1_dphase);
	ay1 = dm(phase1);		{ Last phase }
	ar = ax1 + ay1;			{ New phase=Last phase+Phase Increment}
	ax0 = ar;			{ The phase input to sin is reg ax0 }
	dm(phase1) = ar;		{ Save for next data point }
	call sin;			{ Phase in ax0, Sin returned in ar }
        my0 = dm(sig1_mult);		{ Get amplitude multiplier, in 1.15 }
        mr = ar * my0 (ss);		{ New scaled sine wave }
        dm(signal_save) = mr1;		{ This is the first sine wave, save }

	{ Repeat the steps for a second sine wave }
	ax1=dm(sig2_dphase);
	ay1 = dm(phase2);		{ Last phase }
	ar = ax1 + ay1;			{ New phase=Last phase+Phase Increment}
	ax0 = ar;			{ The phase input to sin is reg ax0 }
	dm(phase2) = ar;		{ Save for next data point }
	call sin;			{ Phase in ax0, Sin returned in ar }
        my0 = dm(sig2_mult);		{ Get multiplier, in 1.15 }
        mr = ar * my0 (ss);		{ New scaled sine wave }
	ay0=dm(signal_save);
	ar=mr1+ay0;			{ Sum 2 sine waves }
        dm(signal_save) = ar;		{ Sine wave signal 1 & 2 in ar }

	{ Next generate a Guassian White Noise }
        call getrnd;			{ 16-bit rand numb, GWN in af }
	ar=pass af;			{ There is no ar=af, this is it }
	my0=dm(noise_mult);		{ The amplitude level }
	mr=ar*my0 (ss);
        ay0 = dm(signal_save);		{ Non-noise part }
        ar = mr1 + ay0;			{ Noise + Two Sigs }

	ax0=dm(off_on);			{ Turn off all signals if off_on=0 }
	af=tstbit 0 of ax0;		{ Check the value }
	if gt jump isa;			{ Jump if signals are on }
	ar=0;				{ Throw them away, if off }
isa:	dm(tx_buf+1) = ar;		{ Send S+N wave to Left D/A (Codec) }
	dm(tx_buf+2) = ar;		{ Right D/A }

	dis sec_reg;			{ Restore the primary registers }
	rti;				{ This undoes the interrupt }

{ LOAD16()  -  This routine can be called from the background to load the data
  in dm(data16) into the hardware shift regs.  Data from dm(data16)
  is unaltered by this routine.  Time to load is about 160 microseconds.  This
  could be much faster, but the delays allow for the rise-time of 
  feedthrough filters on the data, clock and latch wires.
}
load16:	ax0 = dm(data16);		
	dm(sr_data)=ax0;		{ 16 bits of shift data }
	call delay3;			{ Ensure rise of latch pulse }

	cntr = 16;
	do clk_bit until ce;
	    ax0=dm(sr_data);
	    ar=tstbit 15 of ax0;
	    sr0=ax0;
	    sr=lshift sr0 by 1 (lo);	{ Move bits over, status unaffected }
	    dm(sr_data)=sr0;		{ Save shifted data }
	    if eq jump bl4a;
	    ax0 = dm(PFDATA);		{ Data line to 1 }
	    ar = setbit 2 of ax0;
	    dm(PFDATA) = ar;
	    jump bl4e;
bl4a:	    ax0 = dm(PFDATA);		{ Data line to 0 }
	    ar = clrbit 2 of ax0;
	    dm(PFDATA) = ar;

bl4e:	    call delay3;
	    ax0 = dm(PFDATA);		{ Clock line }
	    ar = setbit 0 of ax0;
	    dm(PFDATA) = ar;
	    call delay3;
	    ax0 = dm(PFDATA);		{ Clock fall }
	    ar = clrbit 0 of ax0;
	    dm(PFDATA) = ar;
	    call delay3;
clk_bit:    nop;			{ Don't end loop on a call }

	ax0 = dm(PFDATA);		{ Latch the data }
	ar = setbit 1 of ax0;
	dm(PFDATA) = ar;
	call delay3;
	ax0 = dm(PFDATA);		{ Clear the latch for the next load }
	ar = clrbit 1 of ax0;
	dm(PFDATA) = ar;
	call delay3;
	rts;

{ LCD4()  -  Sends 4 bits to D4-D7 of LCD. These are the lower 4 bits of
  ar.  Bit 7 of ar is sent to RS of the LCD and is 1 for data and 0 for
  command.
}
lcd4:	sr = lshift ar by 8 (hi);
	af = tstbit 15 of sr1;		{ It was bit 7, now bit 15 }
	if ne jump lcd4a;
	ay1 = 0;			{ SR bit (0 = command) }
	jump lcd4b;
lcd4a:	ay1 = b#0000000010000000;	{ SR bit (1=data) }
lcd4b:	ay0 = b#0000111100000000;	{ Data bit mask }
	ar = sr1 and ay0;		{ Clear the SR bit from it's old spot }
	af = ar or ay1;			{ Add SR bit into new spot }
	ax0 = dm(data16);
	ay0 = b#1111000000011111;
	ar = ax0 and ay0;		{ Mask off lcd bits }
	ar = ar or af;
	dm(data16) = ar;
	call load16;			{ Send out 16 bits with enable low }
	ax0 = dm(data16);
	ar = setbit 5 of ax0;		{ Enable bit high to LCD }
	dm(data16) = ar;
	call load16;
	ax0 = dm(data16);
	ar = clrbit 5 of ax0;		{ Enable bit low to LCD }
	dm(data16) = ar;
	call load16;
{ CALL DELAY50; }
	rts;

{ OUT_CMD()  -  Sends the 8 bit command in SR1 to the LCD.
  Note that RS is set to 0 for command by masking, if not already 0 }
out_cmd: dm(temp1) = sr1;		{ Save for second nibble }
	sr = lshift sr1 by -4 (hi);	{ Get high 4 bits }
	ay0 = b#00001111;		{ Mask to get onl these 4 bits }
	ar = sr1 and ay0;
	call lcd4;			{ Send 4 bits of command }
	ar = dm(temp1);			{ Get command again }
	ay0 = b#00001111;
	ar = ar and ay0;
	jump lcd4;

{ OUT_CH()  -  Sends the character in SR1 to the LCD.  }
out_ch:	dm(temp1) = sr1;		{ Save for second nibble }
	sr = lshift sr1 by -4 (hi);	{ Get high 4 bits }
	ay0 = b#10000000;		{ This says it is data }
	ar = sr1 or ay0;
	call lcd4;
	ar = dm(temp1);			{ Get character again }
	ay0 = b#00001111;
	ar = ar and ay0;
	ay0 = b#10000000;		{ This says it is data }
	ar = ar or ay0;
	jump lcd4;

{ N2BCD()  -  Converts the input in AR to ASCII digits in digit[1] msd to
  digit[5] lsb.  A blank is in digit[0] for positive numbers and a - sign
  for negative numbers.
  Signed (-32768, 32767) input. }
n2bcd:	ar = pass ar;
	ax0 = h#20;			{ Blank for plus }
	if ge jump n2s;
	ax0 = h#2d;			{ - sign for negative }
	ar = -ar;
n2s:	dm(digit) =ax0;
	dm(temp1) = ar;			{ The number to be converted to BCD }
	ay0 = 10000;			{ Find the msdigit }
	ay1 = h#30;			{ Count the subtractions }
n2a:	ar = dm(temp1);			{ Test the current reduced number }
	af = ar - ay0;
	if lt jump n2b;			{ Done for this digit }
	ar = ar - ay0;			{ Not done, reduce working number }
	dm(temp1) = ar;
	ar = ay1 + 1;			{ Increase current digit }
	ay1 = ar;			{ This is where it is kept }
	jump n2a;			{ Continue subtractions }
n2b:	dm(digit + 1) = ay1;

	ay0 = 1000;
	ay1 = h#30;
n2c:	ar = dm(temp1);			{ Test the current reduced number }
	af = ar - ay0;
	if lt jump n2d;			{ Done for this digit }
	ar = ar - ay0;			{ Not done, reduce working number }
	dm(temp1) = ar;
	ar = ay1 + 1;			{ Increase current digit }
	ay1 = ar;			{ This is where it is kept }
	jump n2c;			{ Continue subtractions }
n2d:	dm(digit + 2) = ay1;

	ay0 = 100;			{ Find the msdigit }
	ay1 = h#30;			{ Count the subtractions }
n2e:	ar = dm(temp1);			{ Test the current reduced number }
	af = ar - ay0;
	if lt jump n2f;			{ Done for this digit }
	ar = ar - ay0;			{ Not done, reduce working number }
	dm(temp1) = ar;
	ar = ay1 + 1;			{ Increase current digit }
	ay1 = ar;			{ This is where it is kept }
	jump n2e;			{ Continue subtractions }
n2f:	dm(digit + 3) = ay1;

	ay0 = 10;			{ Find the msdigit }
	ay1 = h#30;			{ Count the subtractions }
n2g:	ar = dm(temp1);			{ Test the current reduced number }
	af = ar - ay0;
	if lt jump n2h;			{ Done for this digit }
	ar = ar - ay0;			{ Not done, reduce working number }
	dm(temp1) = ar;
	ar = ay1 + 1;			{ Increase current digit }
	ay1 = ar;			{ This is where it is kept }
	jump n2g;			{ Continue subtractions }
n2h:	dm(digit + 4) = ay1;

	ay1 = h#30;
	ar = ar + ay1;
	dm(digit + 5) = ar;
	rts;

{ KNOB()  -  Reads opto-encoder knob and outputs direction.
  The encoder output is a two bit gray count that has the following
  pattern, where movement to the right is CW rotation:
  Input bit 4: 0011001100 etc.
  Input bit 5: 0110011001 etc.
  These two bits are the encoder "state".
  The previous knob state is kept as the two lower bits of dm(knob_st).
  This previous state is shifted left 2 places (x4) and the new state
  is or'ed in to the first two bits.  This 4 bit number determines the
  entry point to a 16 entry table, encoder[], which determines the
  movement of the encoder, -1, 0 or +1 count.  This value is returned
  in ay0. Status flags are also set.
}
knob:	ay0 = 4;  call inbit;		{ LSB of knob state, returns in ax0 }
	mr1 = 0;			{ In case bit 3 of ax0 = 0 }
	ar = tstbit 3 of ax0;		{ Find out }
	if eq jump kn1;			{ Yes, it is = 0 }
	mr1 = 1;			{ The other case, bit 3 of ax0 = 1 }
kn1:	ay0 = 5;  call inbit;		{ Similar stuff for next to LSB }
	ar = tstbit 3 of ax0;
	if eq jump kn2;
	ar = setbit 1 of mr1;
	mr1 = ar;
kn2:					{ Here with new state in mr1 }
	ar = dm(knob_st);		{ State of knob at last measurement }
	sr = lshift ar by 2 (hi);	{ Move left 2 bits }
	ay0 = sr1;
	ar = mr1 or ay0;		{ 4 bit state }
	dm(knob_st) = mr1;		{ Current state for next time }
	ay0 = ^encoder;			{ The lookup table address }
	ar = ar+ay0;			{ Get location intable }
	i4 = ar;  m4 = 0;  l4 = 0;	{ The 14 index register gives the }
	ay0 = pm(i4, m4);		{   easy way to get the table entry }
	none=pass ay0;			{ Set flags, based on table entry }
	rts;				{ With -1, 0, or +1 in ay0 }

{ send_freq() - Specific for sending the frequency of Sig1 or Sig2,
  putting 0000 to 9999 in positions 2 to 5 of the LCD and Hz in
  6 and 7.
  Arrive with the frequency in Hz in AR, and the ASCII signal number in ax0;
}
send_freq: dm(msgl)=ax0;
	ax0=H#0020;			{ A space }
	dm(msgl+1)=ax0;
	call n2bcd;			{ String now in digit[0] to [5] }
	ax0=dm(digit+2);
	dm(msgl+2)=ax0;
	ax0=dm(digit+3);
	dm(msgl+3)=ax0;
	ax0=dm(digit+4);
	dm(msgl+4)=ax0;
	ax0=dm(digit+5);
	dm(msgl+5)=ax0;
	ax0=H#0048;				{ 'H' }
	dm(msgl+6)=ax0;
	ax0=H#007A;				{ 'z' }
	dm(msgl+7)=ax0;
	{ Fall into send_msg_l() }

{ SEND_MSG_L()  -  Sends the contents of msgl[] to the left 8 character
  positions of the LCD.  }
send_msg_l: sr1 = h#80;
	call out_cmd;			{ Address left 8 characters }
	i0 = ^msgl;			{ Message stored in an array }
	m0 = 1;  l0 = 0;		{ Use a loop to get }
	cntr = 8;			{   all 8 characters sent to LCD}
	do sml1 until ce;
	    sr1 = dm(i0, m0);		{ Get the character }
	    call out_ch;		{ Send it to LCD }
sml1:	    nop;			{ Loops can't end on a call }
	rts;

{ send_lev() - Specific for sending the level of Sig1, Sig2 or Noise,
  putting 000.0 to 999.9 in positions 9 to 13 of the LCD and mV in
  14 and 15.
  Arrive with the level in mV*10 in AR.
}
send_lev: ax0=H#0020;			{ A blank }
	dm(msgr)=ax0;
	call n2bcd;			{ String now in digit[0] to [5] }
	ax0=dm(digit+2);
	dm(msgr+1)=ax0;
	ax0=dm(digit+3);
	dm(msgr+2)=ax0;
	ax0=dm(digit+4);
	dm(msgr+3)=ax0;
	ax0=dm(digit+5);
	dm(msgr+5)=ax0;
	ax0=H#002E;				{ The decimal point }
	dm(msgr+4)=ax0;
	ax0=H#006D;				{ 'm' }
	dm(msgr+6)=ax0;
	ax0=H#0056;				{ 'V' }
	dm(msgr+7)=ax0;
	{ Fall into send_msg_r() }

{ SEND_MSG_R()  -  Sends the contents of msgr[] to the right 8 character
  positions of the LCD.  }
send_msg_r: sr1 = h#a9;
	call out_cmd;	{ Address right 8 characters }
	i0 = ^msgr;
	m0 = 1;  l0 = 0;
	cntr = 8;
	do smr1 until ce;
	    sr1 = dm(i0, m0);
	    call out_ch;
smr1:	    nop;
	rts;

{ INBIT()  -  Gets a hardware input bit.  Bit is selected by ay0 (0,7)
  and the results are returned as ax0 (all PFDATA with the input as bit 3)
  and also as the zero flag in the arithmetic results.  This can be used
  by instructions 'if eq ...' or 'if ne ...'.  }
inbit:	ay1 = b#11111000;		{ First clear 3 bits of data16 }
	ax1 = dm(data16);
	ar = ax1 and ay1;
	ar = ar or ay0;			{ OR in the address of data input }
	dm(data16) = ar;
	call load16;
	ax0 = dm(PFDATA);
	ar = tstbit 3 of ax0;
	rts;

in_sw1:	ay0 = 0;
	jump inbit;

in_sw2:	ay0 = 1;
	jump inbit;

in_sw3:	ay0 = 2;
	jump inbit;

in_sw4:	ay0 = 3;
	jump inbit;

red_on: ar = dm(data16);
	ar = setbit 4 of ar;		{ Turn on red LED }
	dm(data16) = ar;
	jump load16;			{ Send to hardware }

red_off: ar = dm(data16);
	ar = clrbit 4 of ar;		{ Turn off red LED }
	dm(data16) = ar;
	jump load16;			{ Send to hardware }

grn_on: ar = dm(data16);
	ar = setbit 3 of ar;		{ Turn on green LED }
	dm(data16) = ar;
	jump load16;			{ Send to hardware }

grn_off: ar = dm(data16);
	ar = clrbit 3 of ar;		{ Turn off green LED }
	dm(data16) = ar;
	jump load16;			{ Send to hardware }

{ DELAY1  -  A wait of about 1 microsecond. This and delay3 are done
  with multiple NOP's to not use up a place on the loop stack. }
delay1:	nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;
	nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;
	nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;
	rts;

{ DELAY3  -  A wait of about 3 microsecond.  }
delay3:	nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;
	nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;
	nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;
	nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;
	nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;
	nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;
	nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;
	nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;
	nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;
	rts;

{ DELAY50  -  A wait of 50 microseconds. This and delay5m can use loops as
 they are not used except at initialization and loop stack has room. }
delay50: cntr = 1500;
	do dy50 until ce;
dy50:	nop;
	rts;

{ DELAY5M  -  A wait of 5000 microseconds.  }
delay5m: cntr = 4848;
	do dy5 until ce;
	  call delay1;
dy5:	  nop;				{ Don't put call here }
	rts;

{******************************************************************************
 * Main startup initialization.  Many variables and registers require
 * startup initialization, either for operation or to improve simulation.
 * This routine includes the parts that can easily be simulated
 *****************************************************************************}
init0:	ax0 = TIME1;
	dm(clock1) = ax0;
	ax0 = TIME2;
	dm(clock2) = ax0;

	ax0=0;
	dm(sw4)=ax0;			{ Allow sw4 to be a toggle }
	dm(which_fcn)=ax0;		{ Sine wave #1 }
	ax0=100;			{ 10.0 mV }
	dm(sig1_level_mv)=ax0;
	dm(sig2_level_mv)=ax0;
	dm(noise_level_mv)=ax0;
	{ The value in the gain multiplier is 2*100*18981 where the 18981
	  represents the 0.1 mV shifted left 12 places, and the 2 is for the
	  redundant sign bit removal. HI=57 dec, LO=60,648 dec }
	ax0=H#0039;
	dm(sig1_level_hi)=ax0;
	dm(sig2_level_hi)=ax0;
	dm(noise_level_hi)=ax0;
	ax0=H#ECE8;
	dm(sig1_level_lo)=ax0;
	dm(sig2_level_lo)=ax0;
	dm(noise_level_lo)=ax0;
	{ And the multiplier for this is H#39ECE8>>12 or H#039F }
	ax0=H#039F;
	dm(sig1_mult)=ax0;
	dm(sig2_mult)=ax0;
	dm(noise_mult)=ax0;

	ax0=1;
	dm(off_on)=ax0;			{ Start with outputs on }

	ax0=1234;
	dm(sig1_freq)=ax0;
	dm(sig2_freq)=ax0;
	ax0=1684;
	dm(sig1_dphase)=ax0;
	dm(sig2_dphase)=ax0;
	ax0=0;
	dm(phase1)=ax0;
	dm(phase2)=ax0;

	{ i7, l7, i6, l6 devoted to auto-buffering in SPORT0.  Initialized
	  here. These need protection, if used elsewhere in the program.    }
        i7 = ^rx_buf;
        l7 = %rx_buf;
        m7 = 1;
        i6 = ^tx_buf;
        l6 = %tx_buf;
        m6 = 1;
	{ i5, l5, m5 used in next_cmd to load AD1847 regs and then made free. }
        i5 = ^init_cmds;
        l5 = %init_cmds;
        m5 = 1;

        ax0=0xC000; dm(tx_buf)=ax0;    { Mode Change Enable (MCE)  initially }
        ax0=0x0000; dm(tx_buf+1)=ax0;
        ax0=0x0000; dm(tx_buf+2)=ax0;

	{  AD1847  Command Register Initialization.  They are placed into
	   memory here and will be loaded into the AD1847 with interrupt
	   routines in init1.  The details of these registers are given
	   in the AD1847 data sheet.  Note that the destinations are given
	   by the second hex digit and that register addresses b, e and f
	   are not valid, or used.    }
        ax0=0xc002;  dm(init_cmds)=ax0;	   { Left Input control }
        ax0=0xc102;  dm(init_cmds+1)=ax0;  { Right Input control }
        ax0=0xc288;  dm(init_cmds+2)=ax0;  { Left Aux #1 control }
        ax0=0xc388;  dm(init_cmds+3)=ax0;  { Right Aux #1 control }
        ax0=0xc488;  dm(init_cmds+4)=ax0;  { Left Aux #2 control }
        ax0=0xc588;  dm(init_cmds+5)=ax0;  { Right Aux #2 control }
        ax0=0xc680;  dm(init_cmds+6)=ax0;  { Left DAC control }
        ax0=0xc780;  dm(init_cmds+7)=ax0;  { Right DAC control }
        ax0=0xc85c;  dm(init_cmds+8)=ax0;  { Format: 48 KHz, (c850h for 8 KHz)}
        ax0=0xc909;  dm(init_cmds+9)=ax0;  { Interface configuration }
        ax0=0xca00;  dm(init_cmds+10)=ax0; { Pin configuration }
        ax0=0xcc40;  dm(init_cmds+11)=ax0; { Misc Information }
        ax0=0xcd00;  dm(init_cmds+12)=ax0; { Digital Mix control }

{********************* Serial PORT 0 Initialization *************************
 * The following sets up the SPORT0 for autobuffering of 3 receive words
 * (Status, L data, and R data) or 3 transmit words (Command, L data, and
 * R data.)  These are data memory locations, like dm(SPORT0_Autobuf).
 * The actual locations in memory come from the system.k file and the details
 * of the data loaded to these dm locations is given in the Analog Devices
 * "ADSP-2100 Family Users Manual."
 *****************************************************************************}
        ax0= b#0000110101111111;   dm (SPORT0_Autobuf) = ax0;
        ax0 = 0;    dm (SPORT0_RFSDIV) = ax0;
        ax0 = 0;    dm (SPORT0_SCLKDIV) = ax0;
        ax0 = b#1000011000001111;   dm (SPORT0_Control_Reg) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_TX_Channels0) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_TX_Channels1) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_RX_Channels0) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_RX_Channels1) = ax0;

{******************* System and memory initialization ***********************
 * A variety of hardware dependent parameters are are set by the following
 * registers.  The first two are in data memory and ifc, icntl and mstst are
 * three non-memory registers that control hardware operation.
 * The actual locations in memory come from the system.k file, whereas the
 * three non-memory registers are understood directly by the assembler.
 * The details of the data loaded to these dm locations is given in the
 * Analog Devices "ADSP-2100 Family Users Manual."
 *****************************************************************************}

	ax0 = 0;			{ No wait states are used }
	dm(DM_Wait_Reg)=ax0;
        ax0 = b#0001000000000000;	{ Enable SPORT0 }
        dm(System_Control_Reg) = ax0;
        ifc = b#0000000011111111;	{ Clear any pending interrupt }
        nop;
	icntl = 0;			{ Interrupts level sensitive, no nest }
	mstat = b#1000000;		{ Primary data registers }
	ax0 = b#0111111100000111;	{ PF0, 1, 2 out, PF3-7 in }
	dm(PFTYPE)=ax0;
	ax0 = 0;
	dm(PFDATA) = ax0;

	ax0 = RN_SEED;
        dm(seed_lsw) = ax0;             { Arbitrary seed for rn generator }
        dm(seed_msw) = ax0;
	ax0=16384;
	dm(sin_lev)=ax0;
	rts;

{*****************************************************************************
 * Start up initialization of the AD1847 Codec.  This requires some effort to
 * simulate, and it is easiest to skip this part when running the sim21
 * simulator.  This method doesn't support all testing, but is OK for most
 * purposes.
 * This part of the program is the "background" routine during the
 * initialization of the AD1847 Codec.  The SPORT0 Tx interrupt is enabled.
 * The transmission of data to the Codec will program the
 * Codec control registers.  See the AD1847 data sheet for the myriads of
 * details involved.
 *****************************************************************************}
init1:	{ AD 1847 Codec Initialization
	  The stat_flag=0 will prevent reinit of AD1847 from SPORT0 TX ints }
	ax0 = 1;			{ Set flag so control words will be }
	dm(stat_flag) = ax0;		{   be loaded by next_cmd routine }
	imask = b#0001000000;		{ Enable SPORT0 Tx int, disable others}
	ax0 = dm(i6, m6);		{ Start interrupt }
	tx0 = ax0;
check_init:
        ax0 = dm (stat_flag);		{ Wait for entire initialization }
        af = pass ax0;			{   command buffer (init_cmds) to be }
        if ne jump check_init;		{   sent to the codec by next_cmd   }
        ay0 = 2;
check_acih:
        ax0 = dm (rx_buf);		{ Once initialized, wait for codec }
        ar = ax0 and ay0;		{   to come out of autocalibration }
        if eq jump check_acih;		{ Wait for bit set }
check_aci1:
        ax0 = dm (rx_buf);		{ Wait for bit clear }
        ar = ax0 and ay0;
        if ne jump check_aci1;
	idle;				{ Sit and wait for an interrupt }

        ay0 = 0xbf3f;			{ Unmute left DAC }
        ax0 = dm (init_cmds + 6);
        ar = ax0 AND ay0;
        dm (tx_buf) = ar;
        idle;

        ax0 = dm (init_cmds + 7);	{ Unmute right DAC }
        ar = ax0 AND ay0;
        dm (tx_buf) = ar;
        idle;

	ax0 = 0xc901;			{ Clear autocalibration request }
	dm(tx_buf)=ax0;
	idle;

        ax0 = 0x8000;			{ Clear over-range flags }
        dm (tx_buf) = ax0;
        idle;

        ifc = b#00000011111111;		{ Clear any pending interrupt }
        nop;
        imask = b#0000100000;		{ Enable RX0 interrupt, now our only }
	rts;


{************************  NEXT_CMD  ****************************************
 * Transmit Interrupt Service Routine used for Codec initialization
 * DAG 5 registers used here, but are available after this routine is run.
 * The commands are loaded here into the command location dm(tx_buf+0) where
 * they will be transferred to the Codec.
 * Adapted from EZ-KIT Lite routines.
 ****************************************************************************}
next_cmd:
        ena sec_reg;			{ Get other set of registers }
        ax0 = dm (i5, m5);		{ Get next control word and }
        dm (tx_buf+0) = ax0;		{   place in command word }
        ax0 = i5;			{ Now see if finished by comparing }
        ay0 = ^init_cmds;		{   the i5 address with the starting}
        ar = ax0 - ay0;			{   i5 value. Circ buff brings here}
        if gt rti;                  	{ Exit if more control words waiting }
        ax0 = 0x8000;			{ else clear the Mode Change Enable }
        dm (tx_buf+0) = ax0;		{   (MCE) bit in the Codec cntrl word}
        ax0 = 0;			{ Flag the completion by clearing }
        dm (stat_flag) = ax0;		{   stat_flag, stopping calls to here }
	dis sec_reg;
        rti;

{************************  INIT_LCD  ****************************************
 * The following is a prescribed initialization routine for 4-bit
 * operation of the Optrex DMC16117A LCD.
 ****************************************************************************}
init_lcd:
	ar = h#03;
	call lcd4;
	call delay5m;

	ar = h#03;
	call lcd4;
	call delay50;
	call delay50;

	ar = h#03;
	call lcd4;

	ar = h#02;			{ Fcn set, 4 bits }
	call lcd4;

	sr1 = h#28;  call out_cmd;	{ Fcn set, 2 lines, 5x8 }
	sr1 = h#01;  call out_cmd;	{ Clear screen }
	sr1 = h#06;  call out_cmd;	{ Increment, no shift }
	sr1 = h#0C;  jump out_cmd;	{ Display on, no cursor }
	{ End of LCD initialization }

{*************************************************************************
 *      getrnd    Random Number Generator - 8 uniform rn added together
 *                to approximate Gaussian rn.  RSL 14 Feb 96
 * Uniform routine from AD DSP Appl Vol 1.  The linear congruence method is
 * described in Kunth, D. E. "The Art of Computer Programming, Vol 2", Addison
 * Wesle, 1969.  This method multiplies the last random number by a constant
 * a, adds a second constant c and then takes the most significant 16 bits as
 * the new uniformly distributed random number that is uniform on the full
 * 16 bit range.  Taken as signed 1.15 format number this corresponds to -1.0
 * to +32767/32768 with zero mean.
 * After dividing by 8 to prevent numerical overflow, 8 of these uniform
 * random numbers are added together.  An important mathematical principal,
 * called the Central Limit theorem, tells us that as the number of independent
 * numbers added together gets large the result will closely approximate
 * a Gaussian distribution.  Eight gives a good approximation.
 * The sample output in af is in 1.15 format with sigma=0.20412 (sig^2= 1/24)
 * with mean=0;
 * Thus  peak/sigma is 1/.20412 = 4.9 which is sufficient for 99.99998% of
 * all Gaussian samples.
 *
 * Calling value:  dm(seed_msw), dm(seed_lsw)
 * Return value:   af is noise sample
 * Instructions:   134 instruction cycles per output
 *
 *****************************************************************************}
getrnd:	{ The constant a=1664525 decimal, and is represented here as two 16
	  bit values:       }
        my1=25;                         { Upper half of a (1664525/65536) }
        my0=26125;                      { Lower half of a, the remainder }
        af=pass 0;			{ Clear the arithmetic accumulator }
        cntr=8;				{ The number of uniform rn added }
	{ Now loop 8 times to generate a noise sample:   }
        do randloop until ce;		{ Decrease cntr until 0 }
	  sr1=dm(seed_msw);		{ Get the 32 bit seed from last }
	  sr0=dm(seed_lsw);		{   call to this fcn or last loop }

          mr=sr0*my1(uu);		{ 32 bit multiply: a(hi) * x(lo) }
          mr=mr+sr1*my0(uu);            {   and a(hi) * x(lo) + a(lo) X x(hi) }
          si=mr1;			{ Temp storage }
          mr1=mr0;
          mr2=si;
          mr0=h#fffe;                   { c=32767, left-shifted by 1 }
          mr=mr+sr0*my0(uu);            { (above) + a(lo) X x(lo) + c }
          sr=ashift mr2 by 15 (hi);
          sr=sr or lshift mr1 by -1 (hi); { Right-shift by 1 }
          sr=sr or lshift mr0 by -1 (lo); { Now have uniform rn in sr1 }
	  { Use seed_msw and seed_lsw as temporary storage.  This is as
	    efficient as borrowing some unused registers, and less risky when
	    program changes are made. Leaves next seed where we want it, also:}
	  dm(seed_msw)=sr1;		{ Save new seed value, high 16 bits }
	  dm(seed_lsw)=sr0;		{    and low 16 }
	  { Uniform random number still in sr1.  Add on to accumulator:  }
          sr=ashift sr1 by -3 (hi);	{ Divide by 8, i.e., shift right 3 }
randloop: af=sr1+af;			{ Accumulate 8 uniform rn }
	rts;				{ random 16-bit value in af }

{*************************************************************************
 *
 * Sine waves are calculated by a polynomial approximation.  The
 * calculation is done only for 0 to 90 degrees and then the symmetry
 * of the sine wave is used to find the other three quadrants.
 * The phase change from point to point is 1.365333 times the desired
 * frequency in Hz.  The phase change must be rounded to the nearest
 * integer.
 *
 *       Sine Approximation:     ar = Sin(ax0)
 *       Calling Parameters:     ax0 in scaled 1.15 format
 *       Return Values:          ar in 1.15 format
 *       Computation Time:       25 cycles
 * Thanks to Analog Devices Application Handbook #1
 *************************************************************************}
sin:    m5=1;    l5=0;
        i5=^sin_coeff;                  { Pointer to coeff. buffer }
        ay0=H#4000;
        ar=ax0, af=ax0 and ay0;         { Check 2nd or 4th quad. }
        if ne ar=-ax0;                  { If yes, negate input }
        ay0=H#7FFF;
        ar=ar and ay0;                  { Remove sign bit }
        my1=ar;
        mf=ar*my1 (RND), mx1=pm(i5,m5); { mf = input**2 }
        mr=mx1*my1 (SS), mx1=pm(i5,m5); { Start polynomial calculation }
        cntr=3;
        do approx until ce;
                mr=mr+mx1*mf (ss);
approx:         mf=ar*mf (rnd), mx1=pm(i5,m5);
        mr=mr+mx1*mf (ss);
        sr=ashift mr1 by 3 (hi);
        sr=sr or lshift mr0 by 3 (lo);  { Convert to 1.15 format }
        ar=pass sr1;
        if lt ar=pass ay0;              { Saturate if needed }
        af=pass ax0;
        if lt ar=-ar;                   { Negate output if needed }
        rts;

.endmod;
